require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_arcgis_windowsservice',
  dscmeta_resource_friendly_name: 'ArcGIS_WindowsService',
  dscmeta_resource_name: 'ArcGIS_WindowsService',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'ArcGIS',
  dscmeta_module_version: '4.0.0',
  docs: 'The DSC ArcGIS_WindowsService resource type.
         Automatically generated from version 4.0.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_name: {
      type: 'String',
      desc: 'The name for the service',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_builtinaccount: {
      type: "Optional[Enum['LocalSystem', 'LocalService', 'NetworkService']]",
      desc: 'An enumerated value that describes the built in account the service runs under. LocalSystem    LocalService    NetworkService    ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_description: {
      type: 'Optional[String]',
      desc: 'The service description',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_state: {
      type: "Enum['Running', 'Stopped']",
      desc: 'An enumerated value that describes if the service is expected to be running on the machine. Running {default}   Stopped    ',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_dependencies: {
      type: 'Optional[Array[String]]',
      desc: 'A list of service dependencies',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_startuptype: {
      type: "Optional[Enum['Automatic', 'Manual', 'Disabled']]",
      desc: 'An enumerated value that describes the service start type. Automatic    Manual    Disabled    ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_credential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'The optional credentials the service runs under',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_path: {
      type: 'Optional[String]',
      desc: 'The path to the service executable file',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_displayname: {
      type: 'Optional[String]',
      desc: 'The service display name',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
