require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_arcgis_xfirewall',
  dscmeta_resource_friendly_name: 'ArcGIS_xFirewall',
  dscmeta_resource_name: 'ArcGIS_xFirewall',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'ArcGIS',
  dscmeta_module_version: '4.0.0',
  docs: 'The DSC ArcGIS_xFirewall resource type.
         Automatically generated from version 4.0.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_description: {
      type: 'Optional[String]',
      desc: 'Documentation for the Rule',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_direction: {
      type: "Optional[Enum['Inbound', 'Outbound']]",
      desc: 'Direction of the connection',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_displayname: {
      type: 'Optional[String]',
      desc: 'Localized, user-facing name of the Firewall Rule being created',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_remoteport: {
      type: 'Optional[Array[String]]',
      desc: 'Specific Port used for filter. Specified by port number, range, or keyword',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_name: {
      type: 'String',
      desc: 'Name of the Firewall Rule',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: 'Ensure the presence/absence of the resource',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_protocol: {
      type: 'Optional[String]',
      desc: 'Specific Protocol for filter. Specified by name, number, or range',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_localport: {
      type: 'Optional[Array[String]]',
      desc: 'Local Port used for the filter',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_displaygroup: {
      type: 'Optional[String]',
      desc: 'Name of the Firewall Group where we want to put the Firewall Rules',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_service: {
      type: 'Optional[String]',
      desc: 'Specifies the short name of a Windows service to which the firewall rule applies',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_state: {
      type: "Optional[Enum['Enabled', 'Disabled']]",
      desc: 'Enable or disable the supplied configuration',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_profile: {
      type: "Optional[Array[Enum['Any', 'Public', 'Private', 'Domain']]]",
      desc: 'Specifies one or more profiles to which the rule is assigned',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_access: {
      type: "Enum['NotConfigured', 'Allow', 'Block']",
      desc: 'Permit or Block the supplied configuration',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_applicationpath: {
      type: 'Optional[String]',
      desc: 'Path and file name of the program for which the rule is applied',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
